#include "lib/colorCorrection.h"
#include "lib/structure.h"
#include "lib/noiseFbm.h"
#include "lib/waterShading.h"
#include "lib/parallax.h"

vec3 tl(vec3 diffuse)
{
    vec3 col = vec3(7.8, 3.5, 0.0) * 0.5;
    vec3 sat = vec3(dot(col.rgb, vec3(1.0)));
    vec3 result_c = mix(sat, col.rgb, 1.3);

    diffuse += result_c * pow(uv1.x, 5.0);

return diffuse;
}

vec3 tl2(vec3 diffuse)
{
    vec3 col = vec3(7.8, 3.5, 0.0) * 0.1;
    vec3 sat = vec3(dot(col.rgb, vec3(1.0)));
    vec3 result_c = mix(sat, col.rgb, 1.0);

    diffuse += result_c * pow(uv1.x*1.2, 5.0);

return diffuse;
}

void terrain_shader(inout vec4 diffuse, function m, Color n)
{
    highp vec3 np;
    float shadow[2] = float[2](
1.0-min(max((uv1.y-0.90)*100.0,0.0),1.0),
1.0-min(max((uv1.y-0.85)*100.0,0.0),1.0));

    float inverse = 0.0;
    float grad = 0.2;
    float far[3] = float[3](
length(worldpos)/FAR_CHUNKS_DISTANCE * 0.5,
length(worldpos)/RENDER_DISTANCE * 2.0, 
length(worldpos)/RENDER_DISTANCE);
    float bias_torch = (1.0-pow(uv1.x*1.3, 3.0));

#ifdef FANCY
    np = normalize(cross(dFdx(position.zyx),dFdy(position.zyx)));
#endif
    float dx = max(0., np.x), idx = max(0., -np.x);
    float dy = max(0., np.y), idy = max(0., -np.y);
    float dz = max(0., np.z), idz = max(0., -np.z);

    float sp = mix(max(idz,dz), max(max(idz,dz),max(-dy, dx)), m.sunfunc);
    sp = mix(sp, max(idz,dz), m.nightfunc);
    shadow[0] = mix(shadow[0], 1.0, sp);

#ifdef FANCY
#ifdef ALPHA_TEST
    shadow[0] = 1.0-min(max((uv1.y-0.90)*100.0,0.0),1.0);
#endif
#endif
	if(color.a == 0.0){ shadow[0] = 0.0; diffuse.rgb *= 0.9; }

    vec3 bias = mix(n.nooncolor*0.9, mix(tl(diffuse.rgb), tl2(diffuse.rgb), smoothstep(0.0,0.75,uv1.y)*(1.0-m.nightfunc)), shadow[0]);

    vec3 col = mix(mix(n.nooncolor, bias, uv1.x), 
mix(n.suncolor, bias, uv1.x), m.sunfunc);
    col = mix(col, mix(n.nightcolor, tl(diffuse.rgb), uv1.x), m.nightfunc);
    col = mix(col, mix(n.raincolor, tl(diffuse.rgb), uv1.x), m.rainfunc);

    diffuse.rgb = mix(diffuse.rgb, diffuse.rgb*vec3(0.5, 0.5, 0.5), shadow[0]*bias_torch*(1.0-m.rainfunc));

    diffuse.rgb = mix(diffuse.rgb, 
mix(vec3(0.7, 0.8, 1.0) * 0.5, n.fc.rgb, m.nightfunc), idy* 0.6 * (1.0-shadow[1]) * bias_torch * m.rainfunc);

    diffuse.r = (diffuse.r)*(grad + 1.2) + 
(diffuse.g + diffuse.b)*(-grad);
    diffuse.g = (diffuse.g)*(grad + 1.2) + 
(diffuse.r + diffuse.b)*(-grad);
    diffuse.b = (diffuse.b)*(grad + 1.1) + 
(diffuse.r + diffuse.g)*(-grad);
 
    diffuse.rgb = mix(diffuse.rgb, vec3(dot(diffuse.rgb, vec3(0.3))), 0.1); 
   diffuse.rgb *= col;
   diffuse.rgb = acesFilm(diffuse.rgb);

#ifdef FANCY
if(color.r < color.b && color.g > color.r){

    diffuse = water(m, n) * uv1.y * (1.0-m.nightfunc);
    inverse = 1.0;
    diffuse.rgb = mix(diffuse.rgb, diffuse.rgb*vec3(0.4, 0.5, 0.6), shadow[0]*(1.0-m.rainfunc)*bias_torch);
}

#endif
if((n.fc.b > 0.15 && (n.fc.b > n.fc.g*1.2) && (n.fc.b >= n.fc.r*1.8) && (n.fc.g > n.fc.r*1.6)) || (n.fc.g*1.2 >= n.fc.b) && (n.fc.r*1.3 < n.fc.g)){

    float water_w = getWater(2.8*position, worldpos, 0.3);
    diffuse.rgb *= vec3(0.7, 0.85, 1.0) * 0.8;
    diffuse.rgb += water_w * 10.0 * uv1.y * diffuse.rgb * vec3(0.0, 0.7, 1.1) * 3.0 * (1.0-m.nightfunc) * (1.0-inverse);
}

    diffuse.rgb = mix(diffuse.rgb, vec3(0.0, 0.7, 1.0)*n.fc.rgb*0.5, (1.0-uv1.y)*clamp(pow(length(worldpos.xyz*1.5)/FAR_CHUNKS_DISTANCE,1.),0.,1.)*0.8);

   diffuse.rgb = mix(diffuse.rgb, vec3(1.0, 1.0, 1.0)*n.fc.rgb, min(pow(far[1]*0.45, 1.8), 0.7));
}

