// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300

#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
_centroid in highp vec2 uv;
#else
_centroid in vec2 uv;
#endif

#else

varying vec2 uv;

#endif

in highp vec3 pos;
in highp vec4 dpos;

#include "uniformWorldConstants.h"
#include "uniformPerFrameConstants.h"
#include "uniformShaderConstants.h"
#include "uniformRenderChunkConstants.h"
#include "util.h"
#include "lib/structure.h"

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;

//https://www.shadertoy.com/view/Xlc3D2
//modified by me
vec3 sun_moon(function m, Color n)
{
float size = mix(mix(1.8, 1.5, m.sunfunc), 0.9, m.nightfunc);
float b = length(pos.xz);

vec3 res = mix(n.nooncolor, n.suncolor, m.sunfunc);
     res = mix(res, n.nightcolor, m.nightfunc);

    res += max(0.1 / pow(length(pos.xz*size) * 4.0, 8.0), 0.0) + abs(b * 7.0)/35.0;
    res += max(0.01 / pow(length(pos.xz*size) * 1.0, 2.0 / 10.0), .0) + abs(b * 13.0)/25.0;
    res += (max(0.01 / pow(length(pos.xz*size) * 20.0, 1.0 / 2.0), 0.0)*4.0)*4.0;
    res *= exp(0.5 - length(pos.xz)) / 7.0;

return res;
}

void main()
{

#if !defined(TEXEL_AA) || !defined(TEXEL_AA_FEATURE)
	vec4 diffuse = texture2D( TEXTURE_0, uv );
#else
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv );
#endif

vec4 fc = FOG_COLOR;

function m;
   m.sunfunc = pow(max(min(1.0-fc.b * 1.2, 1.0), 0.0), 0.5);
   m.nightfunc = pow(max(min(1.0-fc.r * 1.5, 1.0), 0.0), 1.2);
   m.rainfunc = (1.0 - pow(FOG_CONTROL.y, 11.0));

Color n;
   n.nooncolor = vec3(1.0, 0.8, 0.3);
   n.suncolor = vec3(1.0, 0.6, 0.0) + fc.r;
   n.nightcolor = vec3(1.0) + fc.r + fc.g;
   n.fc = fc;

diffuse.rgb = sun_moon(m, n);

gl_FragColor = diffuse;
gl_FragColor.a = 1.0*(1.0-m.rainfunc);
}
