// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroid.h"

#if __VERSION__ >= 300
	#ifndef BYPASS_PIXEL_SHADER
		#if defined(TEXEL_AA) && defined(TEXEL_AA_FEATURE)
			_centroid in highp vec2 uv0;
			_centroid in highp vec2 uv1;
		#else
			_centroid in vec2 uv0;
			_centroid in vec2 uv1;
		#endif
	#endif
#else
	#ifndef BYPASS_PIXEL_SHADER
		in vec2 uv0;
		in vec2 uv1;
	#endif
#endif

in vec4 color;
in highp vec3 position;
in highp vec3 worldpos;
in highp vec3 viewpos;

#ifdef FOG
in vec4 fogColor;
#endif

#include "uniformWorldConstants.h"
#include "uniformPerFrameConstants.h"
#include "uniformShaderConstants.h"
#include "uniformRenderChunkConstants.h"
#include "util.h"

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;
LAYOUT_BINDING(2) uniform sampler2D TEXTURE_2;
#include "lib/terrainShading.h"

void main()
{
#ifdef BYPASS_PIXEL_SHADER
	gl_FragColor = vec4(0, 0, 0, 0);
	return;
#else 

#if USE_TEXEL_AA
	vec4 diffuse = texture2D_AA(TEXTURE_0, uv0);
#else
	vec4 diffuse = texture2D(TEXTURE_0, uv0);
	vec4 light_tex = texture2D( TEXTURE_1, vec2(0.0,1.0));
#endif
	
#ifdef SEASONS_FAR
	diffuse.a = 1.0;
#endif
function m;
   	m.lp = length(worldpos.xz)/FAR_CHUNKS_DISTANCE;
   	m.nightfunc = max(1.0-min(pow(light_tex.r*1.8,5.0),1.0),0.0);
  	m.sunfunc = pow(clamp((1.0-pow(light_tex.r,5.0))*1.3,0.0,1.0),8.0);
  	m.rainfunc = clamp((1.0-pow(FOG_CONTROL.y+0.1,10.0)),0.0,1.0);
Color n;
  	n.nooncolor = vec3(1.0, 0.98, 0.95);
  	n.suncolor = vec3(0.8,0.6,0.0);
   n.nightcolor = mix(diffuse.rgb, vec3(dot(diffuse.rgb, vec3(0.5))) * vec3(0.3, 0.7, 1.0), 1.0);
  	n.raincolor = vec3(0.35);
  	n.fc = FOG_COLOR;

#if USE_ALPHA_TEST
	#ifdef ALPHA_TO_COVERAGE
	#define ALPHA_THRESHOLD 0.05
	#else
	#define ALPHA_THRESHOLD 0.5
	#endif
	if(diffuse.a < ALPHA_THRESHOLD)
		discard;
#endif

vec4 inColor = color;

#if defined(BLEND)
	diffuse.a *= inColor.a;
#endif

#if !defined(ALWAYS_LIT)
	diffuse *= texture2D(TEXTURE_1, uv1);
#endif

#ifndef SEASONS
	#if !USE_ALPHA_TEST && !defined(BLEND)
		diffuse.a = inColor.a;
	#endif

	vec3 vertex_color= sqrt(color.rgb);
if(color.g + color.g > color.r + color.b){
	vertex_color = color.rgb;
}
	diffuse.rgb *= vertex_color;

#else
	vec2 uv = inColor.xy;
	diffuse.rgb *= mix(vec3(1.0,1.0,1.0), texture2D( TEXTURE_2, uv).rgb*2.0, inColor.b);
	diffuse.rgb *= inColor.aaa;
	diffuse.a = 1.0;
#endif

	terrain_shader(diffuse, m, n);

#ifdef FOG
if((n.fc.b > 0.15 && (n.fc.b > n.fc.g*1.2) && (n.fc.b >= n.fc.r*1.8) && (n.fc.g > n.fc.r*1.6)) || (n.fc.g*1.2 >= n.fc.b) && (n.fc.r*1.3 < n.fc.g)){
	diffuse.rgb = mix(diffuse.rgb, fogColor.rgb, fogColor.a*0.5);
} else {
if(m.rainfunc > 0.0){
	diffuse.rgb = mix(diffuse.rgb, fogColor.rgb, fogColor.a);
}}
#endif

	gl_FragColor = diffuse;
	
#endif // BYPASS_PIXEL_SHADER
}
